"""
        DigiCloud Compute Server Group Service 
"""
'''
    Create Server group
    Delete Server group
    Show Server group
    List Server groups
'''
from rich.prompt import Confirm

from digicloud import schemas
from .base import Lister, ShowOne, Command
from ..cli import parseractions
from ..error_handlers import CLIError
from ..utils import is_tty

class CreateServerGroup(ShowOne):
    schema = schemas.ServerGroupDetails()

    def get_parser(self, prog_name):
        parser = super(CreateServerGroup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help="Server group name"
        )

        parser.add_argument(
            '--policy',
            metavar='<policy>',
            required=True,
            choices=['AFFINITY', 'ANTI_AFFINITY', 'SOFT_AFFINITY', 'SOFT_ANTI_AFFINITY'],
            help=('Server group policy')
        )

        parser.add_argument(
            '--rules',
            metavar='max_server_per_host=<max_server_per_host>',
            dest="rules",
            action=parseractions.MultiKeyValueAction,
            required_keys=['max_server_per_host'],
            default=[],
            help=('Use this property to specify rules. Now only max_server_per_host is allowed'
                  'to be used alongside with anti_affinity policy') 
        )
        return parser

    def _check_arg_validity(self, parsed_args):
        rules = [
            (
                len(parsed_args.rules) and parsed_args.policy.lower() != 'anti_affinity',
                'rules can only be used with anti_affinity policy',
            ),
        ]

        errors = []
        for is_invalid, err_msg in rules:
            if is_invalid:
                errors.append(dict(msg=err_msg))
            if errors:
                raise CLIError(errors)

    def get_data(self, parsed_args):
        self._check_arg_validity(parsed_args)
        
        payload = {
            'name': parsed_args.name,
            'policy': parsed_args.policy
        }

        if parsed_args.rules:
            payload['rules'] = {
                'max_server_per_host': parsed_args.rules[0]['max_server_per_host'],
            }
        data = self.app.session.post('/server-groups', payload)
        return data


class ShowServerGroup(ShowOne):
    schema =  schemas.ServerGroupDetails()

    def get_parser(self, prog_name):
        parser = super(ShowServerGroup, self).get_parser(prog_name)
        parser.add_argument(
            'server_group',
            metavar="<server_group>",
            help='Server group name or ID', 
        )
        return parser

    def get_data(self, parsed_args):
        uri = '/server-groups/%s' % parsed_args.server_group
        data = self.app.session.get(uri)
        return data


class ListServerGroups(Lister):
    schema = schemas.ServerGroupDetails(many=True)

    def get_parser(self, prog_name):
        parser = super(ListServerGroups, self).get_parser(prog_name)
        parser.add_argument(
            '--policy',
            metavar='<policy>',
            choices=['AFFINITY', 'ANTI_AFFINITY', 'SOFT_AFFINITY', 'SOFT_ANTI_AFFINITY'],
            help=('Server group policy')
        )

        return parser

    def get_data(self, parsed_args):
        query_params = {}
        if parsed_args.policy:
            query_params['policy'] = parsed_args.policy

        data = self.app.session.get('/server-groups', params=query_params)
        return data


class DeleteServerGroup(Command):
    def get_parser(self, prog_name):
        parser = super(DeleteServerGroup, self).get_parser(prog_name)
        parser.add_argument(
            'server_group',
            metavar='<server_group>',
            help='Servre group name or ID',
        )
        
        parser.add_argument(
            '--i-am-sure',
            help='Use this switch to bypass confirmation',
            default=None,
            action='store_true',
        )

        return parser

    def take_action(self, parsed_args):
        if not self.confirm_server_group_deletion(parsed_args):
            return 

        uri = '/server-groups/%s' % parsed_args.server_group
        self.app.session.delete(uri)

    def confirm_server_group_deletion(self, parsed_args):
        if parsed_args.i_am_sure:
            return True
        if is_tty():
            serv_group = self.app.session.get('/server-groups/%s' % parsed_args.server_group)
            user_response = Confirm.ask(
                "You're about to delete the server group named [red bold]{}[/red bold]. "
                "Are you sure?".format(
                    serv_group['name']
                ), default=False
            )
            if user_response:
                return True
            self.app.stdout.write("Operation cancelled by user\n")
        else:
            self.app.stderr.write(
                "Unable to perform 'server group delete' operation in non-interactive mode,"
                " without '--i-am-sure' switch\n")
            return False


"""
    DigiCloud IpSec Service.
"""
from rich.prompt import Confirm

from .base import Lister, ShowOne, Command
from ..cli import parseractions
from digicloud import schemas
from ..utils import is_tty


class ListIpSec(Lister):
    """List ipsecs."""
    schema = schemas.IPSecPolicySchema(many=True)

    def get_data(self, parsed_args):
        objects = self.app.session.get('/ipsec-policies')
        return objects


class ShowIpSec(ShowOne):
    """Show ipsec details."""
    schema = schemas.IPSecPolicySchema()

    def get_parser(self, prog_name):
        parser = super(ShowIpSec, self).get_parser(prog_name)
        parser.add_argument(
            'ipsec',
            metavar='<ipsec>',
            help='IpSec name or ID',
        )
        return parser

    def get_data(self, parsed_args):
        uri = '/ipsec-policies/%s' % parsed_args.ipsec
        data = self.app.session.get(uri)
        return data


class DeleteIpSec(Command):
    """Delete ipsec."""

    def get_parser(self, prog_name):
        parser = super(DeleteIpSec, self).get_parser(prog_name)
        parser.add_argument(
            'ipsec',
            metavar='<ipsec>',
            help='IpSec name or ID'
        )
        parser.add_argument(
            '--i-am-sure',
            help='Use this switch to bypass confirmation',
            default=None,
            action='store_true'
        )
        return parser

    def take_action(self, parsed_args):
        if not self.confirm(parsed_args):
            return
        uri = '/ipsec-policies/%s' % parsed_args.ipsec
        self.app.session.delete(uri)

    def confirm(self, parsed_args):
        if parsed_args.i_am_sure:
            return True
        if is_tty():
            ipsec = self.app.session.get('/ipsec-policies/%s' % parsed_args.ipsec)
            user_response = Confirm.ask(
                "You're about to delete a ipsec named "
                "[red bold]{} Are you sure?".format(
                    ipsec['name'],
                ))
            if user_response:
                return True
            self.app.stdout.write("Operation cancelled by user\n")
        else:
            self.app.stderr.write("Unable to perform 'delete ipsec' operation in non-interactive mode,"
                                  " without '--i-am-sure' switch\n")
            return False


class CreateIpSec(ShowOne):
    """Create IpSec."""
    schema = schemas.IPSecPolicySchema()

    def get_parser(self, prog_name):
        parser = super(CreateIpSec, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help='IpSec name'
        )

        parser.add_argument(
            '--auth-algorithm',
            metavar='<auth_algorithm>',
            choices=['sha1', 'sha256', 'sha384', 'sha512'],
            default='sha1',
            help='IpSec auth_algorithm'
        )

        parser.add_argument(
            '--encapsulation-mode',
            metavar='<encapsulation_mode>',
            choices=['tunnel', 'transport'],
            default='tunnel',
            help='IpSec encapsulation mode'
        )

        parser.add_argument(
            '--encryption-algorithm',
            metavar='<encryption_algorithm>',
            choices=['3des', 'aes-128', 'aes-192', 'aes-256'],
            default='aes-128',
            help='IpSec encryption algorithm'
        )

        parser.add_argument(
            '--pfs',
            metavar='<pfs>',
            choices=['group2', 'group5', 'group14'],
            default='group5',
            help='IpSec pfs'
        )

        parser.add_argument(
            '--transform-protocol',
            metavar='<transform_protocol>',
            choices=['esp', 'ah', 'ah-esp'],
            default='esp',
            help='IpSec transform protocol'
        )

        parser.add_argument(
            '--lifetime',
            metavar='units=<units>,value=<value>',
            dest='lifetime',
            action=parseractions.MultiKeyValueAction,
            required_keys=['units', 'value'],
            default=[{'units':'seconds', 'value':'3600'}],
            help='determine lifetime, e.g --lifetime units=seconds,value=3600'
        )

        return parser

    def get_data(self, parsed_args):
        lifetime = parsed_args.lifetime[0]
        lifetime['value'] = int(lifetime['value'])
        payload = {
            'name': parsed_args.name,
            'auth_algorithm': parsed_args.auth_algorithm,
            'encapsulation_mode': parsed_args.encapsulation_mode,
            'encryption_algorithm': parsed_args.encryption_algorithm,
            'pfs': parsed_args.pfs,
            'transform_protocol': parsed_args.transform_protocol,
            'lifetime': lifetime,
        }
        payload = self.schema.load(payload)
        data = self.app.session.post('/ipsec-policies', payload)
        return data



"""
    DigiCloud Compute Instance Service.
"""
from rich.prompt import Confirm

from digicloud import schemas
from .base import Lister, ShowOne, Command
from ..utils import is_tty

class ListDatastore(Lister):
    """List datastore."""
    schema = schemas.DbaasDatastoreList(many=True)

    def get_data(self, parsed_args):
        data = self.app.session.get('/dbaas/datastores')

        datastores = []
        for datastore in data:
            for version in datastore['versions']:
                datastores.append({
                    'name': datastore['name'],
                    'version': version['name'],
                })

        return datastores


class ListInstance(Lister):
    """List database instance."""
    schema = schemas.DbaasInstanceList(many=True)

    def get_data(self, parsed_args):
        data = self.app.session.get('/dbaas/instances')
        return data


class CreateInstance(ShowOne):
    """Create database Instance."""
    schema = schemas.DbaasInstanceDetails()

    def get_parser(self, prog_name):
        parser = super(CreateInstance, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help='Database Instance name'
        )
        parser.add_argument(
            '--instance-type',
            required=True,
            metavar='<instance_type>',
            help='InstanceType name or ID'
        )
        parser.add_argument(
            '--volume-size',
            required=True,
            type=int,
            metavar='<volume_size>'
        )
        parser.add_argument(
            '--volume-type',
            required=False,
            metavar='<volume_type>',
            choices=['SSD', 'ULTRA_DISK'],
            type=lambda value: str(value).upper().replace("-", "_"),
            help=(
                'Optionally you can set the type of root volume, '
                'could be SSD or ULTRA_DISK case insensitive, the default is SSD'
            )
        )
        parser.add_argument(
            '--network',
            required=True,
            metavar='<network>',
            help='Network name or ID'
        )
        parser.add_argument(
            '--datastore-name',
            metavar='<datastore_name>',
            required=True,
            help='Datastore name',
        )
        parser.add_argument(
            '--datastore-version',
            metavar='<datastore_version>',
            required=True,
            help='Datastore version',
        )
        parser.add_argument(
            '--is-public',
            required=False,
            help='Whether the database service is exposed to the public',
            action = 'store_true'
        )
        parser.add_argument(
            '--allow-cidr',
            required=False,
            metavar='<allow_cidr>',
            help='A list of IPv4 that restrict access to the database service. 0.0.0.0/0 is used by default if this parameter is not provided.',
        )

        return parser

    def get_data(self, parsed_args):
        payload = {
            'name': parsed_args.name,
            'instance_type': parsed_args.instance_type,
            'network': parsed_args.network,
            'datastore_name': parsed_args.datastore_name,
            'datastore_version': parsed_args.datastore_version,
            'volume_type': parsed_args.volume_type,
            'volume_size': parsed_args.volume_size,
        }
        if parsed_args.is_public:
            payload['is_public'] = parsed_args.is_public
        if parsed_args.allow_cidr:
            payload['allow_cidr'] = parsed_args.allow_cidr

        data = self.app.session.post('/dbaas/instances', payload)
        return data


class ShowInstance(ShowOne):
    """Show database instance details."""
    schema = schemas.DbaasInstanceDetails()

    def get_parser(self, prog_name):
        parser = super(ShowInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help='Database instance name or ID',
        )
        return parser

    def get_data(self, parsed_args):
        uri = '/dbaas/instances/%s' % parsed_args.instance
        data = self.app.session.get(uri)
        return data


class DeleteInstance(Command):
    """Delete a  database instance."""

    def get_parser(self, prog_name):
        parser = super(DeleteInstance, self).get_parser(prog_name)

        parser.add_argument(
            'instance',
            metavar='<instance>',
            help='Instance name or ID',
        )
        parser.add_argument(
            '--i-am-sure',
            help='Use this switch to bypass confirmation',
            default=None,
            action='store_true'
        )
        return parser

    def take_action(self, parsed_args):
        if not self.confirm(parsed_args):
            return
        uri = '/dbaas/instances/%s' % parsed_args.instance
        self.app.session.delete(uri)

    def confirm(self, parsed_args):
        if parsed_args.i_am_sure:
            return True
        if is_tty():
            instance = self.app.session.get('/dbaas/instances/%s' % parsed_args.instance)
            user_response = Confirm.ask(
                "You're about to delete a database instance named "
                "[red bold]{}[/red bold]. "
                "Are you sure?".format(
                    instance['name'],
                ))
            if user_response:
                return True
            self.app.stdout.write("Operation cancelled by user\n")
        else:
            self.app.stderr.write("Unable to perform 'delete database instance' operation in non-interactive mode,"
                                  " without '--i-am-sure' switch\n")
            return False


class ListDatabase(Lister):
    """List database instance."""
    schema = schemas.DbaasDatabaseList(many=True)

    def get_parser(self, prog_name):
        parser = super(ListDatabase, self).get_parser(prog_name)

        parser.add_argument(
            '--instance',
            required=True,
            metavar='<instance>',
            help='Instance name or ID'
        )

        return parser

    def get_data(self, parsed_args):
        uri = '/dbaas/instances/%s/databases' % (parsed_args.instance)
        data = self.app.session.get(uri)
        return data


class CreateDatabase(ShowOne):
    """Create dbass db."""
    schema = schemas.DbaasDatabaseDetails()

    def get_parser(self, prog_name):
        parser = super(CreateDatabase, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help='Database name'
        )
        parser.add_argument(
            '--instance',
            required=True,
            metavar='<instance>',
            help='Instance name or ID'
        )

        return parser

    def get_data(self, parsed_args):
        payload = {
            'name': parsed_args.name,
        }

        uri = '/dbaas/instances/%s/databases' % (parsed_args.instance)
        data = self.app.session.post(uri, payload)
        return data


class DeleteDatabase(Command):
    """Delete dbass db."""

    def get_parser(self, prog_name):
        parser = super(DeleteDatabase, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help='Database Instance name'
        )
        parser.add_argument(
            '--instance',
            required=True,
            metavar='<instance>',
            help='Instance name or ID'
        )
        parser.add_argument(
            '--i-am-sure',
            help='Use this switch to bypass confirmation',
            default=None,
            action='store_true'
        )

        return parser

    def take_action(self, parsed_args):
        if not self.confirm(parsed_args):
            return
        uri = '/dbaas/instances/%s/databases/%s' % (parsed_args.instance, parsed_args.name)
        self.app.session.delete(uri)

    def confirm(self, parsed_args):
        if parsed_args.i_am_sure:
            return True
        if is_tty():
            user_response = Confirm.ask(
                "You're about to delete a database, "
                "Are you sure?")
            if user_response:
                return True
            self.app.stdout.write("Operation cancelled by user\n")
        else:
            self.app.stderr.write("Unable to perform 'delete database' operation in non-interactive mode,"
                                  " without '--i-am-sure' switch\n")
            return False

class ListUser(Lister):
    """List database users."""
    schema = schemas.DbaasUserList(many=True)

    def get_parser(self, prog_name):
        parser = super(ListUser, self).get_parser(prog_name)

        parser.add_argument(
            '--instance',
            required=True,
            metavar='<instance>',
            help='Instance name or ID'
        )

        return parser

    def get_data(self, parsed_args):
        uri = '/dbaas/instances/%s/users' % (parsed_args.instance)
        data = self.app.session.get(uri)
        return data


class CreateUser(ShowOne):
    """Create dbass user."""
    schema = schemas.DbaasUserDetails()

    def get_parser(self, prog_name):
        parser = super(CreateUser, self).get_parser(prog_name)
        parser.add_argument(
            '--instance',
            metavar='<instance>',
            help='instance name or id'
        )
        parser.add_argument(
            '--username',
            metavar='<username>',
            help='Database username'
        )
        parser.add_argument(
            '--password',
            metavar='<password>',
            help='Database password'
        )
        parser.add_argument(
            '--host',
            required=False,
            metavar='<host>',
            help='host'
        )
        parser.add_argument(
            '--databases',
            dest='databases',
            help='list of databases',
            required=False
        )

        return parser

    def get_data(self, parsed_args):
        payload = {
            'name': parsed_args.username,
            'password': parsed_args.password,
        }
        if parsed_args.host:
            payload['host'] = parsed_args.host
        if parsed_args.databases:
            payload['databases'] =  parsed_args.databases.split(',')

        uri = '/dbaas/instances/%s/users' % (parsed_args.instance)
        data = self.app.session.post(uri, payload)
        return data


class DeleteUser(Command):
    """Delete dbass user."""

    def get_parser(self, prog_name):
        parser = super(DeleteUser, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help='Database username'
        )
        parser.add_argument(
            '--instance',
            required=True,
            metavar='<instance>',
            help='Instance name or ID'
        )
        parser.add_argument(
            '--i-am-sure',
            help='Use this switch to bypass confirmation',
            default=None,
            action='store_true'
        )

        return parser

    def take_action(self, parsed_args):
        if not self.confirm(parsed_args):
            return
        uri = '/dbaas/instances/%s/users/%s' % (parsed_args.instance, parsed_args.name)
        self.app.session.delete(uri)

    def confirm(self, parsed_args):
        if parsed_args.i_am_sure:
            return True
        if is_tty():
            user_response = Confirm.ask(
                "You're about to delete a database user, "
                "Are you sure?")
            if user_response:
                return True
            self.app.stdout.write("Operation cancelled by user\n")
        else:
            self.app.stderr.write("Unable to perform 'delete database user' operation in non-interactive mode,"
                                  " without '--i-am-sure' switch\n")
            return False


"""
    DigiCloud IKE Service.
"""
from rich.prompt import Confirm

from .base import Lister, ShowOne, Command
from ..cli import parseractions
from digicloud import schemas
from ..utils import is_tty


class ListIKE(Lister):
    """List ikes."""
    schema = schemas.IKEPolicySchema(many=True)

    def get_data(self, parsed_args):
        objects = self.app.session.get('/ike-policies')
        return objects


class ShowIKE(ShowOne):
    """Show ike details."""
    schema = schemas.IKEPolicySchema()

    def get_parser(self, prog_name):
        parser = super(ShowIKE, self).get_parser(prog_name)
        parser.add_argument(
            'ike',
            metavar='<ike>',
            help='IKE name or ID',
        )
        return parser

    def get_data(self, parsed_args):
        uri = '/ike-policies/%s' % parsed_args.ike
        data = self.app.session.get(uri)
        return data


class DeleteIKE(Command):

    def get_parser(self, prog_name):
        parser = super(DeleteIKE, self).get_parser(prog_name)
        parser.add_argument(
            'ike',
            metavar='<ike>',
            help='IKE name or ID'
        )
        parser.add_argument(
            '--i-am-sure',
            help='Use this switch to bypass confirmation',
            default=None,
            action='store_true'
        )
        return parser

    def take_action(self, parsed_args):
        if not self.confirm(parsed_args):
            return
        uri = '/ike-policies/%s' % parsed_args.ike
        self.app.session.delete(uri)

    def confirm(self, parsed_args):
        if parsed_args.i_am_sure:
            return True
        if is_tty():
            ike = self.app.session.get('/ike-policies/%s' % parsed_args.ike)
            user_response = Confirm.ask(
                "You're about to delete a ike named "
                "[red bold]{} Are you sure?".format(
                    ike['name'],
                ))
            if user_response:
                return True
            self.app.stdout.write("Operation cancelled by user\n")
        else:
            self.app.stderr.write("Unable to perform 'delete ike' operation in non-interactive mode,"
                                  " without '--i-am-sure' switch\n")
            return False


class CreateIKE(ShowOne):
    """Create IKE."""
    schema = schemas.IKEPolicySchema()

    def get_parser(self, prog_name):
        parser = super(CreateIKE, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help='IKE name'
        )

        parser.add_argument(
            '--auth-algorithm',
            metavar='<auth_algorithm>',
            choices=['sha1', 'sha256', 'sha384', 'sha512'],
            default='sha1',
            help='IKE auth_algorithm'
        )

        parser.add_argument(
            '--encryption-algorithm',
            metavar='<encryption_algorithm>',
            choices=['3des', 'aes-128', 'aes-192', 'aes-256'],
            default='aes-128',
            help='IKE encryption algorithm'
        )

        parser.add_argument(
            '--pfs',
            metavar='<pfs>',
            choices=['group2', 'group5', 'group14'],
            default='group5',
            help='IKE pfs'
        )

        parser.add_argument(
            '--ike-version',
            metavar='<ike_version>',
            choices=['v1', 'v2', ],
            default='v1',
            help='IKE version'
        )

        parser.add_argument(
            '--lifetime',
            metavar='units=<units>,value=<value>',
            dest='lifetime',
            action=parseractions.MultiKeyValueAction,
            required_keys=['units', 'value'],
            default=[{'units':'seconds', 'value':'3600'}],
            help='determine lifetime, e.g --lifetime units=seconds,value=3600'
        )

        return parser

    def get_data(self, parsed_args):
        lifetime = parsed_args.lifetime[0]
        lifetime['value'] = int(lifetime['value'])
        payload = {
            'name': parsed_args.name,
            'auth_algorithm': parsed_args.auth_algorithm,
            'ike_version': parsed_args.ike_version,
            'encryption_algorithm': parsed_args.encryption_algorithm,
            'pfs': parsed_args.pfs,
            'ike_version': parsed_args.ike_version,
            'lifetime': lifetime,
        }
        payload = self.schema.load(payload)
        data = self.app.session.post('/ike-policies', payload)
        return data
